@@ -1,40 +1,7 @@
 #!/usr/bin/env python
 
-"""
-ancient-genomes: Estimate coverage, depth and other properties for each genome
-                 in each synthetic community
-
-Usage:
-  ancient-genomes [options] <config>
-  ancient-genomes -h | --help
-  ancient-genomes --version
-
-Options:
-  <config>       Config parameters
-  -d --debug     Debug mode (no subprocesses; verbose output)
-  -h --help      Show this screen.
-  --version      Show version.
-
-Description:
-  Simulating ancient reads for each taxon in each synthetic community
-
-  config
-  ------
-  * tab-delimited
-  * must contain 3 columns
-    * "Community" = community ID (ie., sample ID)
-    * "Taxon" = taxon name
-    * "Perc_rel_abund" = percent relative abundance of the taxon
-
-  Output
-  ------
-  * A JSON file with the read properties for the selected ancient genomes
-  * A TSV file with the read abundances for the ancient/modern genomes in each
-    synthetic community
-"""
-
-# import
-# batteries
+from distutils.log import info
+from statistics import mode
 from docopt import docopt
 import logging
 import pandas as pd
@@ -44,6 +11,8 @@ import numpy as np
 import json
 from pathlib import Path
 import sys
+import math
+from scipy.stats import lognorm
 
 # application
 from MGSIM import SimReads
@@ -54,7 +23,7 @@ import tqdm
 from collections import OrderedDict
 from aMGSIM import __version__
 
-debug = None
+debug = False
 
 
 def exceptionHandler(
@@ -68,10 +37,15 @@ def exceptionHandler(
         # raise
         debug_hook(exception_type, exception, traceback)
     else:
-        print("{}: {}".format(exception_type.__name__, exception))
+        print(f"{exception_type.__name__}: {exception}")
+        print("\n*** Error: Please use --debug to see full traceback.")
 
 
-logging.basicConfig(format="%(asctime)s - %(message)s", level=logging.DEBUG)
+logging.basicConfig(
+    level=logging.DEBUG,
+    format="%(levelname)s ::: %(asctime)s ::: %(message)s",
+    datefmt="%Y-%m-%d %H:%M:%S",
+)
 
 
 def obj_dict(obj):
@@ -87,6 +61,7 @@ def process_genome(
     max_len,
     coverage,
     library,
+    read_length_freqs,
     selected_genomes,
     genome_comp,
     n_reads,
@@ -100,6 +75,7 @@ def process_genome(
         max_len=max_len,
         coverage=coverage,
         library=library,
+        read_length_freqs=read_length_freqs,
         # seqSys=seqSys,
         selected_genomes=selected_genomes,
         genome_comp=genome_comp,
@@ -121,6 +97,7 @@ class Genome:
         max_len,
         coverage,
         library,
+        read_length_freqs,
         n_reads,
         *args,
         **kwargs,
@@ -135,10 +112,34 @@ class Genome:
         self.genome_size = genome["Genome_size"]
         # test if key is in dictionary
 
-        if "TaxId" in genome.keys():
-            self.taxId = genome["TaxId"]
+        if "Accession" in genome.keys():
+            self.accession = genome["Accession"]
         else:
-            self.taxId = None
+            self.accession = None
+
+        # if "Read_length" in genome_comp.keys():
+        #     rl = genome_comp[genome_comp["Taxon"] == self.taxon].iloc[0]["Read_length"]
+        #     mode_len_ancient = rl
+        #     mode_len_modern = rl
+
+        # if "Read_length_std" in genome_comp.keys():
+        #     stddev = genome_comp[genome_comp["Taxon"] == self.taxon].iloc[0][
+        #         "Read_length_std"
+        #     ]
+        #     stddev_len_ancient = stddev
+        #     stddev_len_modern = stddev
+        # else:
+        #     stddev_len_ancient = mode_len_ancient - min_len
+        #     stddev_len_modern = mode_len_modern - min_len
+
+        if "Read_length_min" in genome_comp.keys():
+            min_len = genome_comp[genome_comp["Taxon"] == self.taxon].iloc[0][
+                "Read_length_min"
+            ]
+        if "Read_length_max" in genome_comp.keys():
+            max_len = genome_comp[genome_comp["Taxon"] == self.taxon].iloc[0][
+                "Read_length_max"
+            ]
         # self.library = library
         # self.read_length = max_len
         # self.seqSys = seqSys
@@ -186,6 +187,7 @@ class Genome:
                 min_len=min_len,
                 max_len=max_len,
                 dist=dist,
+                read_length_freqs=read_length_freqs,
                 mode_len=mode_len_ancient,
                 rnd_seed=rnd_seed,
             )
@@ -242,6 +244,7 @@ class Genome:
                 max_len=max_len,
                 dist=dist,
                 mode_len=mode_len_modern,
+                read_length_freqs=read_length_freqs,
                 rnd_seed=rnd_seed,
             )
             if library == "se":
@@ -295,6 +298,7 @@ class Genome:
                 min_len=min_len,
                 max_len=max_len,
                 dist=dist,
+                read_length_freqs=read_length_freqs,
                 mode_len=mode_len_ancient,
                 rnd_seed=rnd_seed,
             )
@@ -302,6 +306,7 @@ class Genome:
                 min_len=min_len,
                 max_len=max_len,
                 dist=dist,
+                read_length_freqs=read_length_freqs,
                 mode_len=mode_len_modern,
                 rnd_seed=rnd_seed,
             )
@@ -345,39 +350,58 @@ class Genome:
                 self.coverage_enforced = True
             self.seq_depth = seq_depth
 
-    def random_fragments(self, min_len, max_len, dist, mode_len, rnd_seed):
-        n_frags = int(1e6)
-        # n_frags = max_len - min_len + 1
-        stddev = mode_len - min_len
-        sigma, scale = f.lognorm_params(mode=mode_len, stddev=stddev)
-        mu = np.log(scale)
-        dist_params = {"mean": mu, "sigma": sigma}
-        # drawing relative abundances from the user-defined distribution
-        freq_dist = f.get_freq_dist(dist, dist_params)
-        dist_freqs = freq_dist(size=n_frags)
-        dist_freqs = [int(x) for x in dist_freqs]
-        # freqs = np.sort(dist_freqs / sum(dist_freqs) * 100)[::-1]
-        # making a series for the taxa
-        lengths = pd.Series(dist_freqs)
-        lengths = lengths[(lengths >= min_len) & (lengths <= max_len)]
-        lengths = lengths.value_counts().sort_index()
-        freqs = list(lengths / sum(lengths))
-        lens = np.multiply(list(lengths.index), freqs).sum()
-        avg_len = float(lens / sum(freqs))
-        # print("Lengths: {}", list(lengths.index))
-        # print("Freqs: {}", list(lengths/sum(lengths)))
-        # print("Sizes: {}".format(sizes))
-        # print("Avg size: {}".format(avg_size))
-        # exit()
-        frags = {
-            "fragments": {"length": list(lengths.index), "freq": freqs},
-            "dist_params": {
-                "mu": float(mu),
-                "sigma": float(sigma),
-                "rnd_seed": rnd_seed,
-            },
-            "avg_len": avg_len,
-        }
+    def random_fragments(
+        self, min_len, max_len, dist, read_length_freqs, mode_len, rnd_seed
+    ):
+        if read_length_freqs:
+            lengths = read_length_freqs[self.accession]["length"]
+            freqs = read_length_freqs[self.accession]["freq"]
+            lens = np.multiply(lengths, freqs).sum()
+            avg_len = float(lens / sum(freqs))
+            frags = {
+                "fragments": {
+                    "length": lengths,
+                    "freq": freqs,
+                },
+                "dist_params": {
+                    "scale": None,
+                    "sigma": None,
+                    "rnd_seed": None,
+                },
+                "avg_len": avg_len,
+            }
+        else:
+            n_frags = int(1e6)
+            stddev = mode_len - min_len
+            sigma, scale = f.lognorm_params(mode=mode_len, stddev=stddev)
+            mu = np.log(scale)
+            dist_params = {"mean": mu, "sigma": sigma}
+            # drawing relative abundances from the user-defined distribution
+            freq_dist = f.get_freq_dist(dist, dist_params)
+            dist_freqs = freq_dist(size=n_frags)
+            dist_freqs = [int(x) for x in dist_freqs]
+            # freqs = np.sort(dist_freqs / sum(dist_freqs) * 100)[::-1]
+            # making a series for the taxa
+            lengths = pd.Series(dist_freqs)
+            lengths = lengths[(lengths >= min_len) & (lengths <= max_len)]
+            lengths = lengths.value_counts().sort_index()
+            freqs = list(lengths / sum(lengths))
+            lens = np.multiply(list(lengths.index), freqs).sum()
+            avg_len = float(lens / sum(freqs))
+            # print("Lengths: {}", list(lengths.index))
+            # print("Freqs: {}", list(lengths/sum(lengths)))
+            # print("Sizes: {}".format(sizes))
+            # print("Avg size: {}".format(avg_size))
+            # exit()
+            frags = {
+                "fragments": {"length": list(lengths.index), "freq": freqs},
+                "dist_params": {
+                    "scale": float(scale),
+                    "sigma": float(sigma),
+                    "rnd_seed": rnd_seed,
+                },
+                "avg_len": avg_len,
+            }
         return frags
 
     def __str__(self):
@@ -605,7 +629,8 @@ def applyParallel(dfGrouped, func, nproc, parms):
 
 
 def _get_read_abundances(self, n_reads):
-    df = pd.DataFrame()
+    # df = pd.DataFrame()
+    dfs = []
     data_dict = OrderedDict()
     for p in self:
         if p.fragments_ancient is None:
@@ -614,28 +639,37 @@ def _get_read_abundances(self, n_reads):
             data_dict["frag_type"] = "ancient"
             data_dict["seq_depth"] = 0
             _df = pd.DataFrame([data_dict], columns=data_dict.keys())
-            df = df.append(_df, ignore_index=True, sort=False)
+            # df = df.append(_df, ignore_index=True, sort=False)
+            # df = f.concat_df([df, _df])
+            dfs.append(_df)
         else:
             data_dict["comm"] = str(p.comm)
             data_dict["taxon"] = p.taxon
             data_dict["frag_type"] = "ancient"
             data_dict["seq_depth"] = p.fragments_ancient["seq_depth"]
             _df = pd.DataFrame([data_dict], columns=data_dict.keys())
-            df = df.append(_df, ignore_index=True, sort=False)
+            # df = df.append(_df, ignore_index=True, sort=False)
+            # df = f.concat_df([df, _df])
+            dfs.append(_df)
         if p.fragments_modern is None:
             data_dict["comm"] = str(p.comm)
             data_dict["taxon"] = p.taxon
             data_dict["frag_type"] = "modern"
             data_dict["seq_depth"] = 0
             _df = pd.DataFrame([data_dict], columns=data_dict.keys())
-            df = df.append(_df, ignore_index=True, sort=False)
+            # df = df.append(_df, ignore_index=True, sort=False)
+            # df = f.concat_df([df, _df])
+            dfs.append(_df)
         else:
             data_dict["comm"] = str(p.comm)
             data_dict["taxon"] = p.taxon
             data_dict["frag_type"] = "modern"
             data_dict["seq_depth"] = p.fragments_modern["seq_depth"]
             _df = pd.DataFrame([data_dict], columns=data_dict.keys())
-            df = df.append(_df, ignore_index=True, sort=False)
+            # df = df.append(_df, ignore_index=True, sort=False)
+            # df = f.concat_df([df, _df])
+            dfs.append(_df)
+    df = f.concat_df(dfs)
     df["seq_depth_rounded"] = df.groupby("comm")["seq_depth"].transform(
         round_to_nreads, n_reads=n_reads
     )
@@ -700,7 +734,7 @@ def round_to_nreads(number_set, n_reads, digit_after_decimal=0):
     The down-side is that the results won't be accurate, but they are never accurate anyway:)
     """
     unround_numbers = [
-        x / float(sum(number_set)) * n_reads * 10 ** digit_after_decimal
+        x / float(sum(number_set)) * n_reads * 10**digit_after_decimal
         for x in number_set
     ]
     decimal_part_with_index = sorted(
@@ -708,7 +742,7 @@ def round_to_nreads(number_set, n_reads, digit_after_decimal=0):
         key=lambda y: y[1],
         reverse=True,
     )
-    remainder = n_reads * 10 ** digit_after_decimal - sum(
+    remainder = n_reads * 10**digit_after_decimal - sum(
         [int(x) for x in unround_numbers]
     )
     index = 0
@@ -716,41 +750,51 @@ def round_to_nreads(number_set, n_reads, digit_after_decimal=0):
         unround_numbers[decimal_part_with_index[index][0]] += 1
         remainder -= 1
         index = (index + 1) % len(number_set)
-    return [int(x) / float(10 ** digit_after_decimal) for x in unround_numbers]
+    return [int(x) / float(10**digit_after_decimal) for x in unround_numbers]
 
 
-def main(args):
+log = logging.getLogger("my_logger")
+
+
+def get_ancient_genomes(args):
+    logging.basicConfig(
+        level=logging.DEBUG,
+        format="%(levelname)s ::: %(asctime)s ::: %(message)s",
+        datefmt="%Y-%m-%d %H:%M:%S",
+        force=True,
+    )
     # simulating reads
     global debug
-    if args["--debug"]:
+    if args.debug:
         debug = True
     else:
-        debug = None
+        debug = False
 
-    sys.excepthook = exceptionHandler
+    log.setLevel(logging.DEBUG if debug else logging.INFO)
+
+    # sys.excepthook = exceptionHandler
 
     # simulating reads
-    args = f.validate_schema(args, d.schema_init_ag, debug)
-    config = f.get_config(
-        config=args["<config>"], schema=d.ag_schema_config, debug=debug
-    )
+    cfg = vars(args)
+    cfg = f.validate_schema(cfg, d.schema_init_ag, debug)
+    config = f.get_config(config=cfg["config"], schema=d.ag_schema_config, debug=debug)
 
     # Test that both genome composition of abundance table are given
-    if not config["abund_table"] and not config["genome_comp"]:
+    if not config["abund-table"] and not config["genome-comp"]:
         raise IOError(
             "You need to provide an abundance table and/or genome composition table"
         )
 
-    if config["abund_table"] and not config["genome_comp"]:
-        abund_table = config["abund_table"]
+    if config["abund-table"] and not config["genome-comp"]:
+        abund_table = config["abund-table"]
         genome_comp = False
         filename = abund_table
-    elif config["genome_comp"] and not config["abund_table"]:
-        genome_comp = config["genome_comp"]
+    elif config["genome-comp"] and not config["abund-table"]:
+        genome_comp = config["genome-comp"]
         filename = genome_comp
     else:
-        genome_comp = config["genome_comp"]
-        abund_table = config["abund_table"]
+        genome_comp = config["genome-comp"]
+        abund_table = config["abund-table"]
         filename = abund_table
 
     if config["library"] == "single":
@@ -763,7 +807,7 @@ def main(args):
 
     max_len = d.seqSys_props[seqSys][library]
 
-    # logging.info('Set maximum fragment length to {}nt based on the seqSys: {}...'.format(
+    # log.info('Set maximum fragment length to {}nt based on the seqSys: {}...'.format(
     #     max_len, seqSys))
 
     # Define different variables
@@ -775,26 +819,23 @@ def main(args):
     mode_len_modern = config["mode-len-modern"]
     n_reads = config["num-reads"]
     coverage = {"min": config["coverage"][0], "max": config["coverage"][1]}
+    read_length_freqs = config["read-length-freqs"]
 
     if read_len > max_len:
-        raise ValueError(
-            "Read length longer than the one allowed by {}...".format(seqSys)
-        )
+        raise ValueError(f"Read length longer than the one allowed by {seqSys}...")
 
     if read_len < min_len:
-        raise ValueError(
-            "Read length shorter than the minimum defined ({})...".format(min_len)
-        )
+        raise ValueError(f"Read length shorter than the minimum defined ({min_len})...")
 
     # load tables
-    logging.info("Loading abundace table...")
+    log.info("Loading abundace table...")
     abund_table = SimReads.load_abund_table(abund_table)
 
-    logging.info("Loading abundance genomes...")
-    genome_table = f.load_genome_table(config["genome_table"])
+    log.info("Loading abundance genomes...")
+    genome_table = f.load_genome_table(config["genome-table"], nproc)
 
     if genome_comp:
-        logging.info("Loading genome composition table...")
+        log.info("Loading genome composition table...")
         genome_comp = pd.read_csv(genome_comp, sep="\t")
         genome_comp = genome_comp.rename(columns={"Coverage": "coverage_ancient"})
     else:
@@ -802,9 +843,13 @@ def main(args):
 
     df = abund_table.merge(genome_table, on=["Taxon"])
 
-    if "TaxId" in df.columns:
+    if read_length_freqs:
+        log.info("Loading read length frequencies from JSON...")
+        read_length_freqs = f.load_read_length_freqs(read_length_freqs)
+
+    if "Accession" in df.columns:
         genomes = df[
-            ["Community", "TaxId", "Taxon", "Perc_rel_abund", "Rank", "Genome_size"]
+            ["Community", "Accession", "Taxon", "Perc_rel_abund", "Rank", "Genome_size"]
         ].to_dict("records")
     else:
         genomes = df[
@@ -813,7 +858,7 @@ def main(args):
 
     # We need to add to the taxon the proportion of ancient
     # We need to take Community,
-    logging.info("Selecting random genomes...")
+    log.info("Selecting random genomes...")
     df_anc = df[["Community", "Taxon", "Perc_rel_abund", "Rank", "Genome_size"]].copy()
     # df_anc['Perc_rel_abund'] = df_anc.groupby('Community')['Perc_rel_abund'].transform(round_to_100_percent)
     grouped = df_anc.groupby("Community")
@@ -841,11 +886,14 @@ def main(args):
             grouped, func=select_genomes, nproc=nproc, parms=parms
         )
 
-    logging.info(
-        "Generating random fragment lengths from {} distribution with modes {} and {}...".format(
-            dist, mode_len_ancient, mode_len_modern
+    if read_length_freqs:
+        log.info(f"Generating random fragment lengths from JSON file")
+        log.info(f"::: Using empirical read lengths")
+    else:
+        log.info(f"Generating random fragment lengths from {dist} distribution")
+        log.info(
+            f"::: Using read lengths with modes [{mode_len_ancient}/{mode_len_modern}]"
         )
-    )
 
     func = partial(
         process_genome,
@@ -856,6 +904,7 @@ def main(args):
         max_len=read_len,
         coverage=coverage,
         library=library,
+        read_length_freqs=read_length_freqs,
         # seqSys=seqSys,
         selected_genomes=selected_genomes,
         n_reads=n_reads,
@@ -867,13 +916,21 @@ def main(args):
     else:
         p = Pool(nproc)
         ancient_genomes_data = list(
-            tqdm.tqdm(p.imap_unordered(func, genomes), total=len(genomes))
+            tqdm.tqdm(
+                p.imap_unordered(func, genomes),
+                total=len(genomes),
+                leave=False,
+                ncols=100,
+                desc=f"Genomes processed",
+            )
         )
+        p.close()
+        p.join()
         # ancient_genomes_data = p.map(func, genomes)
     file_name = Path(filename.name).stem
     dir_name = Path(filename.name).parent
 
-    logging.info("Refinning read abundances...")
+    log.info("Refinning read abundances...")
     ancient_genomes = {}
     ancient_genomes["data"] = refine_abundances(ancient_genomes_data, n_reads=n_reads)
     # ancient_genomes["data"] = ancient_genomes_data
@@ -884,7 +941,6 @@ def main(args):
         "n_reads": n_reads,
         "date": datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
     }
-
     df_tsv = _get_read_abundances(ancient_genomes_data, n_reads=n_reads)
 
     suffix = ".json"
@@ -892,7 +948,7 @@ def main(args):
 
     #
     suffix_tsv = ".tsv"
-    out_file_tsv = Path(dir_name, "{}_read-abundances".format(file_name)).with_suffix(
+    out_file_tsv = Path(dir_name, f"{file_name}_read-abundances").with_suffix(
         suffix_tsv
     )
     df_tsv.to_csv(path_or_buf=out_file_tsv, sep="\t", header=True, index=False)
@@ -901,16 +957,7 @@ def main(args):
         ancient_genomes, default=obj_dict, ensure_ascii=False, indent=4
     )
     # ancient_genomes_json = jsonpickle.encode(ancient_genomes)
-    logging.info("Saving results to: {} & {}...".format(out_file, out_file_tsv))
+    log.info(f"Saving results to: {out_file} & {out_file_tsv}...")
 
     with open(out_file, "w", encoding="utf-8") as outfile:
         print(ancient_genomes_json, file=outfile)
-
-
-def opt_parse(args=None):
-    version = "Version: " + __version__
-    if args is None:
-        args = docopt(__doc__, version=version)
-    else:
-        args = docopt(__doc__, version=version, argv=args)
-    main(args)
