@@ -24,7 +24,7 @@ def generate_random_sample_name():
     return "".join(random.choices(string.ascii_lowercase + string.digits, k=10))
 
 
-debug = None
+debug = False
 
 FWD_ADPT = "AGATCGGAAGAGCACACGTCTGAACTCCAGTCACCGATTCGATCTCGTATGCCGTCTTCTGCTTG"
 REV_ADPT = "AGATCGGAAGAGCGTCGTGTAGGGAAAGAGTGTAGATCTCGGTGGTCGCCGTATCATTT"
@@ -47,9 +47,20 @@ IUPACAmbiguousDNA = {
     "N",
 }
 
-default_filter_values_mdmg = {"d_max": 0.1, "phi": 500, "q": 0.25}
-default_filter_values_woltka = {"breadth_exp_ratio": 0.5, "coverage_mean": 0.1}
-tax_ranks = ["superkingdom", "phylum", "class", "order", "family", "genus", "species"]
+default_filter_values_mdmg = {"D_max": 0.1, "phi": 500, "q": 0.25}
+default_filter_values_filterBAM = {"breadth_exp_ratio": 0.5, "coverage_mean": 0.1}
+tax_ranks = [
+    "superkingdom",
+    "lineage",
+    "kingdom",
+    "phylum",
+    "class",
+    "order",
+    "family",
+    "genus",
+    "species",
+    "subspecies",
+]
 genome_selection_options = ["random", "most_abundant", "least_abundant"]
 # schema_init_ag = {
 #     '<abund_table>': Use(open, error='<abund_table> should be readable'),
@@ -78,18 +89,18 @@ genome_selection_options = ["random", "most_abundant", "least_abundant"]
 # }
 
 schema_init_ag = {
-    "<config>": Use(open, error="config should be readable"),
+    "config": Use(open, error="config should be readable"),
     object: object,
 }
 
 schema_init_w = {
-    "<config>": Use(open, error="config should be readable"),
+    "config": Use(open, error="config should be readable"),
     object: object,
 }
 
 schema_init_ar = {
-    "<config>": Use(open, error="config should be readable"),
-    "<genome_table>": Use(open, error="genome_table should be readable"),
+    "config": Use(open, error="config should be readable"),
+    "genome_table": Use(open, error="genome_table should be readable"),
     object: object,
 }
 
@@ -106,15 +117,20 @@ schema_art_config = {"art_config": Use(open, error="art_config should be readabl
 
 
 ag_schema_config = {
-    "genome_table": And(Use(open, error="genome_table should be readable")),
-    Optional("abund_table", default=False): Or(
+    "genome-table": And(Use(open, error="genome-table should be readable")),
+    Optional("abund-table", default=False): Or(
         bool, And(Use(open)), error="Problem finding the abundance table file"
     ),
-    Optional("genome_comp", default=False): Or(
+    Optional("genome-comp", default=False): Or(
         bool,
         And(Use(open)),
         error="Problem finding the custom genome composition table file",
     ),
+    Optional("read-length-freqs", default=False): Or(
+        bool,
+        And(Use(open)),
+        error="Problem finding the read length frequencies JSON file",
+    ),
     Optional("cpus", default=1): Or(
         None,
         And(Use(int), lambda n: 0 < n < cpu_count() - 1),
@@ -184,27 +200,23 @@ ag_schema_config = {
 }
 
 w_schema_config = {
-    "genome_paths": And(
+    "genome-paths": And(
         Use(lambda x: f.get_open_func(x)(x, "rt")),
         error="genome_paths should be readable",
     ),
-    "woltka_stats": And(
+    "filterBAM-stats": And(
         Use(lambda x: f.get_open_func(x)(x, "rt")),
         error="Problem finding the abundance table file",
     ),
-    "woltka_profiling_results": And(
-        Use(lambda x: f.get_open_func(x)(x, "rt")),
-        error="Problem finding the custom genome composition table file",
-    ),
-    "mdmg_results": And(
+    "mdmg-results": And(
         Use(lambda x: f.get_open_func(x)(x, "rt")),
         error="mdmg_results should be readable",
     ),
-    Optional("woltka_stats_filter", default=default_filter_values_woltka,): And(
+    Optional("filterBAM-stats-filter", default=default_filter_values_filterBAM,): And(
         Use(
             lambda x: f.check_filter_conditions(
                 x,
-                default_filter_values_woltka,
+                default_filter_values_filterBAM,
                 filters=["breadth_exp_ratio", "coverage_mean"],
             )
         ),
@@ -214,30 +226,39 @@ w_schema_config = {
     #     Use(lambda x: f.get_open_func(x)(x, "rt")),
     #     error="mdmg_misincorporation should be readable",
     # ),
-    Optional("mdmg_filter_conditions", default=default_filter_values_mdmg,): And(
+    Optional("mdmg-filter-conditions", default=default_filter_values_mdmg,): And(
         Use(
             lambda x: f.check_filter_conditions(
-                x, default_filter_values_mdmg, filters=["d_max", "phi", "q"]
+                x,
+                default_filter_values_mdmg,
+                filters=[
+                    "D_max",
+                    "lambda_LR",
+                    "q",
+                    "phi",
+                    "Bayesian_D_max",
+                    "Bayesian_z",
+                ],
             )
         ),
         error="There's a problem with the filter. Please check the keys in the dict are valid and that all the values are >= 0",
     ),
-    Optional("taxonomic_rank", default="genus"): And(
+    Optional("taxonomic-rank", default="genus"): And(
         str,
         lambda x: x in tax_ranks,
         error=f"Invalid taxonomic ranks. Pick one valid from {', '.join(str(x) for x in tax_ranks)}",
     ),
-    Optional("max_genomes_nondamaged", default=10): Or(
+    Optional("max-genomes-nondamaged", default=10): Or(
         None,
         And(Use(int), lambda max_genomes_nondamaged: max_genomes_nondamaged > 0),
         error="The number of non-damaged genomes selected should be greater than 0",
     ),
-    Optional("max_genomes_damaged", default=10): Or(
+    Optional("max-genomes-damaged", default=10): Or(
         None,
         And(Use(int), lambda max_genomes_damaged: max_genomes_damaged > 0),
         error="The number of damaged genomes selected should be greater than 0",
     ),
-    Optional("max_genomes_nondamaged_selection", default="most_abundant"): And(
+    Optional("max-genomes-nondamaged-selection", default="most_abundant"): And(
         str,
         lambda x: x in genome_selection_options,
         error=f"Invalid option. Pick one valid from {', '.join(str(x) for x in tax_ranks)}",
@@ -249,7 +270,7 @@ w_schema_config = {
             cpu_count() - 1
         ),
     ),
-    Optional("sample_name", default=generate_random_sample_name()): Or(
+    Optional("sample-name", default=generate_random_sample_name()): Or(
         None,
         And(
             Use(str),
@@ -296,13 +317,13 @@ ar_schema_config = {
         # Optional('seq_depth', default=1e-5): Or(None, And(Use(float), lambda x: x > 0),
         #                                         error='Sequencing depth has to be larger than 0'),
         Optional(
-            "tmp_dir",
+            "tmp-dir",
             default=os.path.join(
                 os.getcwd(), datetime.datetime.now().strftime(".tmp-%Y%m%d%H%M%S")
             ),
         ): And(Use(str)),
         Optional(
-            "output_dir",
+            "output-dir",
             default=os.path.join(
                 os.getcwd(), datetime.datetime.now().strftime("sim_reads-%Y%m%d%H%M%S")
             ),
@@ -314,9 +335,12 @@ ar_schema_config = {
                 cpu_count() - 1
             ),
         ),
+        Optional("remove-adapters", default=True): And(
+            bool, error="The key dist should be True/False"
+        ),
     },
     And("fragSim"): {
-        And("ancient_genomes"): And(
+        And("ancient-genomes"): And(
             str,
             lambda x: os.path.exists(x),
             error="Cannot open the file with the ancient genomes information.",
@@ -384,43 +408,43 @@ ar_schema_config = {
             error="Problem finding the second-read quality profile",
         ),
     },
-    And("AdapterRemoval"): {
-        Optional("--adapter1", default=FWD_ADPT): And(
-            And(Use(str), lambda x: IUPACAmbiguousDNA.issuperset(x)),
-            error="Forward adapter is not a valid sequence",
-        ),
-        Optional("--adapter2", default=REV_ADPT): And(
-            And(Use(str), lambda x: IUPACAmbiguousDNA.issuperset(x)),
-            error="Reverse adapter is not a valid sequence",
-        ),
-        Optional("--minlength", default=30): Or(
-            None,
-            And(Use(int), lambda n: 0 <= n),
-            error="Minimum read length should be larger than 0",
-        ),
-        Optional("--trimns", default=True): And(
-            bool, error="The key trimns should be True/False"
-        ),
-        Optional("--collapse", default=True): And(
-            bool, error="The key collapse should be True/False"
-        ),
-        Optional("--trimqualities", default=True): And(
-            bool, error="The key trimqualities should be True/False"
-        ),
-        Optional("--minquality", default=20): Or(
-            None,
-            And(Use(int), lambda n: 0 <= n),
-            error="Minimum read quality value should be larger than 0",
-        ),
-        Optional("--minadapteroverlap", default=30): Or(
-            None,
-            And(Use(int), lambda n: 0 < n),
-            error="Minimum overlap length should be larger or equal than 0",
-        ),
-        Optional("--preserve5p", default=True): And(
-            bool, error="The key preserve5p should be True/False"
-        ),
-    },
+    # And("AdapterRemoval"): {
+    #     Optional("--adapter1", default=FWD_ADPT): And(
+    #         And(Use(str), lambda x: IUPACAmbiguousDNA.issuperset(x)),
+    #         error="Forward adapter is not a valid sequence",
+    #     ),
+    #     Optional("--adapter2", default=REV_ADPT): And(
+    #         And(Use(str), lambda x: IUPACAmbiguousDNA.issuperset(x)),
+    #         error="Reverse adapter is not a valid sequence",
+    #     ),
+    #     Optional("--minlength", default=30): Or(
+    #         None,
+    #         And(Use(int), lambda n: 0 <= n),
+    #         error="Minimum read length should be larger than 0",
+    #     ),
+    #     Optional("--trimns", default=True): And(
+    #         bool, error="The key trimns should be True/False"
+    #     ),
+    #     Optional("--collapse", default=True): And(
+    #         bool, error="The key collapse should be True/False"
+    #     ),
+    #     Optional("--trimqualities", default=True): And(
+    #         bool, error="The key trimqualities should be True/False"
+    #     ),
+    #     Optional("--minquality", default=20): Or(
+    #         None,
+    #         And(Use(int), lambda n: 0 <= n),
+    #         error="Minimum read quality value should be larger than 0",
+    #     ),
+    #     Optional("--minadapteroverlap", default=30): Or(
+    #         None,
+    #         And(Use(int), lambda n: 0 < n),
+    #         error="Minimum overlap length should be larger or equal than 0",
+    #     ),
+    #     Optional("--preserve5p", default=True): And(
+    #         bool, error="The key preserve5p should be True/False"
+    #     ),
+    # },
 }
 
 seqSys_props = {
