@@ -1,8 +1,8 @@
 import numpy as np
 import ruamel.yaml
-from collections import OrderedDict
+from collections import OrderedDict, ChainMap
 from distutils.spawn import find_executable
-from functools import partial
+from functools import partial, reduce
 from itertools import chain
 import sys
 import re
@@ -12,6 +12,10 @@ from mimetypes import guess_type
 from Bio import SeqIO
 import pandas as pd
 from multiprocessing import Pool
+import gzip
+import shutil
+import json
+import ujson
 
 
 def str2dict(s):
@@ -309,6 +313,7 @@ def load_genome_table(in_file, nproc=1):
         p = Pool(nproc)
         df["Genome_size"] = p.map(_genome_size, [x for i, x in df.iterrows()])
         p.close()
+        p.join()
     else:
         df["Genome_size"] = [_genome_size(x) for i, x in df.iterrows()]
 
@@ -334,6 +339,11 @@ def _genome_size(x):
     return bp
 
 
+def load_read_length_freqs(file):
+    file = ujson.load(file)
+    return reduce(lambda d, src: d.update(src) or d, file, {})
+
+
 def get_compression_type(filename):
     """
     Attempts to guess the compression (if any) on a file using the first few bytes.
@@ -400,4 +410,26 @@ def concat_df(frames):
         # Flatten and save to df_dict
         df_dict[col] = fast_flatten(extracted)
     df = pd.DataFrame.from_dict(df_dict)[COLUMN_NAMES]
-    return df
\ No newline at end of file
+    return df
+
+
+def initializer(init_data):
+    global parms
+    parms = init_data
+
+
+# from https://stackoverflow.com/questions/53751050/python-multiprocessing-understanding-logic-behind-chunksize/54032744#54032744
+def calc_chunksize(n_workers, len_iterable, factor=4):
+    """Calculate chunksize argument for Pool-methods.
+    Resembles source-code within `multiprocessing.pool.Pool._map_async`.
+    """
+    chunksize, extra = divmod(len_iterable, n_workers * factor)
+    if extra:
+        chunksize += 1
+    return chunksize
+
+
+def unzip_files(infile, outfile):
+    with gzip.open(infile, "rb") as f_in:
+        with open(outfile, "wb") as f_out:
+            shutil.copyfileobj(f_in, f_out)
